if( ! window['_u'] && window['_'])
    _u = _.noConflict();

(function($, _) {

    $(function() {
        if( ! window['qcontact_utils'] )
            qcontact_utils = {};

        google.maps.Map.prototype.setCenterWithOffset= function(latlng, offsetX, offsetY) {
            var map = this;
            var ov = new google.maps.OverlayView();

            ov.onAdd = function() {
                var proj = this.getProjection();
                var aPoint = proj.fromLatLngToContainerPixel(latlng);
                aPoint.x = aPoint.x+offsetX;
                aPoint.y = aPoint.y+offsetY;
                map.setCenter(proj.fromContainerPixelToLatLng(aPoint));
            };

            ov.draw = function() {};

            ov.setMap(this);
        };

         pixelToLatlng = function(xcoor, ycoor, map) {
            var ne = map.getBounds().getNorthEast();
            var sw = map.getBounds().getSouthWest();
            var projection = map.getProjection();
            var topRight = projection.fromLatLngToPoint(ne);
            var bottomLeft = projection.fromLatLngToPoint(sw);

            var scale = 1 << map.getZoom();

            var newLatlng = projection.fromPointToLatLng(new google.maps.Point(xcoor / scale + bottomLeft.x, ycoor / scale + topRight.y));

            return newLatlng;
        };

        google.maps.Marker.prototype.getUnique = function() {
            return this._unique;
        };

        google.maps.Marker.prototype.setUnique = function(unique) {
            this._unique = unique;

            return this;
        };


        google.maps.Map.prototype.getMarkers = function() {
            return _.get(this, 'markers', {});
        };

        google.maps.Map.prototype.addMarker = function(marker) {
            if( !_.has(marker, '_unique') )
                marker.setUnique(Date.now());

            if(! _.has(this, 'markers'))
                this.markers = {};

            this.markers[marker.getUnique()] = marker;

            return marker;
        };

        google.maps.Map.prototype.updateMarker = function(marker) {
           if(_.has(this.markers, marker.getUnique()) )
               this.markers[marker.getUnique()] = marker;

            return marker;
        };

        google.maps.Map.prototype.getMarker = function(unique) {
            return _.get(this.markers, unique, false);
        };

        google.maps.Map.prototype.removeMarker = function(marker) {
            var markers = this.markers;

            if(_.has(this.markers, marker.getUnique()) )
                delete this.markers[marker.getUnique()];

            return this;
        };


        /**
         * Initialize map .
         *
         * @param element_id
         * @param callback
         * @param options
         * @returns {mi.Map}
         */
        qcontact_utils.init_gmap = function (element_id, callback, options) {
            options = _.merge({
                center: new google.maps.LatLng(44.5403, -78.5463),
                zoom: 8,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                zoomControl: true,
                scaleControl: false,
                scrollwheel: false,
                draggable: true,
                disableDoubleClickZoom: true,
                mapTypeControl: false,
                streetViewControl: false
            }, options);

            var map = new google.maps.Map(
                document.getElementById(element_id),
                options
            );

            if (callback)
                callback(element_id, map);

            return map;
        };

        /**
         * Add marker to the map .
         *
         * @param map
         * @param lat
         * @param lng
         * @param options
         * @param is_admin
         * @returns {mi.Marker}
         */
        qcontact_utils.add_gmap_marker = function(map, lat, lng, options, is_admin) {
            options = _.merge({
                draggable: true,
                position: {
                    lat: lat,
                    lng: lng
                },
                map: map
            }, options);

            if(! is_admin)
                options.draggable = false;

            var marker = new google.maps.Marker(options);

            map.addMarker(marker);

            if( is_admin ) {

                var is_draggable = _.get(options, 'draggable', false);

                /**
                 * Check if marker is draggable than update marker position .
                 *
                 */
                if( is_draggable && is_admin )
                    google.maps.event.addListener(marker, 'dragend', function() {
                        var container = map.getDiv();

                        map.updateMarker(marker);

                        var markers = map.getMarkers(),
                            toUpdateMarkers = [];

                        _.each(markers, function(v, k) {
                            toUpdateMarkers.push({
                                lat: v.position.lat(),
                                lng: v.position.lng(),
                                title: v.title
                            });
                        });

                        $(container).attr('data-markers' , JSON.stringify(toUpdateMarkers));
                    });
            }

            var can_create_info = false;

            if( is_admin ) {
                can_create_info = true;

                var template = $('<div class="qcontact_form_gmap"><label>Add your address:</label><textarea class="form-control">'+_.get(marker, 'title', '')+'</textarea><button name="remove-marker" class="btn remove-marker" title="Remove Marker"><i class="zmdi zmdi-close"></i></button>&nbsp;&nbsp;<button name="update-marker" class="btn update-marker" title="Save Marker"><i class="zmdi zmdi-floppy"></i></button></div>');
            } else {
                if(_.has(options, 'title') ) {
                    can_create_info = true;

                    var template = $('<div class="qcontact_form_gmap"><p>'+ _.get(marker, 'title', '')+'</p></div>');
                }
            }

            if( can_create_info ) {
                var infowindow = new google.maps.InfoWindow();

                infowindow.setContent(template[0]);

                google.maps.event.addListener(marker, 'click', function() {
                    infowindow.open(map, marker);
                });

                if( is_admin ) {

                    var removeBtn   = template.find('button.remove-marker')[0];
                    var updateBtn   = template.find('button.update-marker')[0];
                    var textarea    = $(template, document).find('textarea').text();

                    /**
                     * On delete marker event .
                     *
                     */
                    google.maps.event.addDomListener(removeBtn, "click", function() {
                        marker.setMap(null);

                        var container = map.getDiv();

                        map.removeMarker(marker);

                        var markers = map.getMarkers(),
                            toUpdateMarkers = [];

                        _.each(markers, function(v, k) {
                            toUpdateMarkers.push({
                                lat: v.position.lat(),
                                lng: v.position.lng(),
                                title: v.title
                            });
                        });

                        $(container).attr('data-markers' , JSON.stringify(toUpdateMarkers));
                    });


                    /**
                     * On update marker event .
                     *
                     */
                    google.maps.event.addDomListener(updateBtn, "click", function(e) {
                        var address = $(e.target).closest('div').find('textarea').val();

                        marker.title = address;
                        map.updateMarker(marker);
                        var container = map.getDiv();

                        var markers = map.getMarkers(),
                            toUpdateMarkers = [];


                        _.each(markers, function(v, k) {
                            toUpdateMarkers.push({
                                lat: v.position.lat(),
                                lng: v.position.lng(),
                                title: v.title
                            });
                        });

                        $(container).attr('data-markers' , JSON.stringify(toUpdateMarkers));

                        infowindow.close()
                    });
                }
            }

            return marker;
        };

        /**
         * Add markers to the map .
         *
         * @param map
         * @param markers
         * @param is_admin
         * @returns {*}
         */
        qcontact_utils.add_gmap_markers = function(map, markers, is_admin) {
            var instances = [];

            $.each(markers, function(k, v) {
                var options = {
                    title: _.get(v, 'title', '')
                };

                var created = qcontact_utils.add_gmap_marker(map, v.lat, v.lng, options, is_admin);

                instances.push(created);
            });

            return instances;
        };

        /**
         * Load map and setUp all configurations .
         *
         * @param selector
         * @param is_admin
         * @param callback
         */
        qcontact_utils.load_map = function(selector, is_admin, callback) {
            var lat = selector.data('lat'),
                lng = selector.data('lng'),
                zoom = selector.data('zoom'),
                default_zoom = 8;

            var run_admin_callback = function(map) {

                /**
                 * Adding listener for each event and store data to container data attributes .
                 *
                 */
                map.addListener('idle', function() {
                    var center = map.getCenter();

                    selector.attr('data-zoom', map.getZoom());
                    selector.attr('data-lng', center.lng());
                    selector.attr('data-lat', center.lat());
                });
            };

            if ( (lat && lng) && ( lat != '' && lng != '' ) ) {
                var map = qcontact_utils.init_gmap(selector.attr('id'), callback, {
                    center: new google.maps.LatLng(
                        lat, lng
                    ),
                    zoom: (zoom !== undefined) ? zoom : default_zoom,
                    draggable: is_admin
                });

                selector.data('map', map);

                var markers = _.get(selector.data(), 'markers', []);

                if( markers.length )
                    qcontact_utils.add_gmap_markers(map, markers, is_admin);

                if( is_admin )
                    run_admin_callback(map);

            } else {
                var html5Options = {
                    enableHighAccuracy: true,
                    timeout: 6000,
                    maximumAge: 0
                };

                geolocator.locate(function(data) {
                    var coordinates = data.coords,
                        markers = _.get(selector.data(), 'markers', []);

                    var map = qcontact_utils.init_gmap(selector.attr('id'), callback, {
                        center: new google.maps.LatLng(
                            coordinates['latitude'], coordinates['longitude']
                        ),
                        zoom: (zoom !== undefined) ? zoom : default_zoom,
                        draggable: is_admin
                    });

                    selector.attr('data-lng', coordinates['longitude']);
                    selector.attr('data-lat', coordinates['latitude']);

                    selector.data('map', map);

                    if( markers.length )
                        qcontact_utils.add_gmap_markers(map, markers, is_admin);

                    if( is_admin )
                        run_admin_callback(map);

                }, function() {
                    alert('Unable to detect location!')
                }, true, html5Options, 'map-canvas');
            }
        };

    });

})(jQuery, _u);