<?php

namespace Builder\App\Application\Bootstrap\Helpers;

/**
 * Die and dump
 */
use Builder\App\Application;

function dd($var, $die = true) {
    echo '<pre style="color: #cc351c">';
    var_dump($var);
    $die ? die : null;
}

/**
 * Dump code .
 * @param $var
 */
function d($var) {
    dd($var, false);
}

/**
 * Return application instance .
 *
 * @param null $slug
 * @return Application
 */
function app($slug = null) {
    $app = Application::getInstance();

    if( $slug )
        return $app->make($slug);

    return $app;
}

/**
 * Get value from config .
 * @param $template
 * @param array $vars
 * @return
 */
function view($template, $vars = array()) {
    $view = app('view');

    return $view->{$template}($vars);
}

/**
 * Render settings for specific page .
 *
 * @param $sections
 * @return mixed
 */
function sections($sections = array()) {
    $fields = Application::getInstance()
        ->make('fields');

    return $fields->render($sections);
}

/**
 * Return full path to assets
 *
 * @param $src
 * @return string
 */
function assets($src) {
    return plugins_url( 'assets/' . $src, realpath(__DIR__ . '/../') );
}

/**
 * Sluggify title .
 *
 * @param $text
 * @return string
 */
function sluggify($text) {
    // replace non letter or digits by -
    $text = preg_replace('~[^\\pL\d]+~u', '-', $text);

    // trim
    $text = trim($text, '-');

    // transliterate
    $text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);

    // lowercase
    $text = strtolower($text);

    // remove unwanted characters
    $text = preg_replace('~[^-\w]+~', '', $text);

    if (empty($text)) {
        return 'n-a';
    }

    return $text;
}

/**
 * Get a settings by key .
 *
 * @param $key
 * @param $default
 * @return mixed
 */
function settings($key, $default = null) {
    $settings = app('settings');

    return $settings->get($key, $default);
}


/**
 * Flash message
 *
 * @param $message
 * @param $type
 * @param array $options
 * @return mixed
 */
function notify($message = null, $type = null, array $options = array()) {
    $notify = app('notify');

    if( $message )
       $notify->flash($message, $type, $options);

    return $notify;
}