<?php

use Builder\App\Application\Bootstrap\Helpers as h;
use Builder\Includes\Exceptions\ValidationException;
use ReCaptcha\ReCaptcha;

return array(
    /**
     * Register providers which have loading .
     */
    'providers' => array(
        'front' => array(
            'Builder\App\Providers\SettingsProvider',
            'Builder\App\Providers\ActionProvider',
            'Builder\App\Providers\ShortcodeProvider',
            'Builder\App\Providers\AssetsProvider',
            'Builder\App\Providers\LanguageProvider',
            'Builder\App\Providers\NotifyProvider',
            'Builder\App\Providers\ViewProvider'
        ),
        'admin' => array(
            'Builder\App\Providers\LanguageProvider',
            'Builder\App\Providers\MenuProvider',
            'Builder\App\Providers\AssetsProvider',
            'Builder\App\Providers\SettingsProvider',
            'Builder\App\Providers\FieldsProvider',
            'Builder\App\Providers\ViewProvider',
            'Builder\App\Providers\TemplateProvider',
            'Builder\App\Providers\PostsProvider',
            'Builder\App\Providers\MediaProvider',
            'Builder\App\Providers\ShortcodeProvider',
            'Builder\App\Providers\ActionProvider',
            'Builder\App\Providers\FilterProvider',
            'Builder\App\Providers\NotifyProvider'
        )
    ),

    /**
     * Declare all assets .
     */
    'assets' => array(
        'aliases' => array(
            'back-css' => h\assets('css/backend.css'),
            'front-css' => h\assets('css/frontend.css'),

            'bootstrap-js' => array(
                'src' => h\assets('js/bootstrap.min.js'),
                'dep' => array('jquery', 'jquery-ui-draggable', 'jquery-ui-droppable', 'jquery-ui-sortable', 'jquery-ui-tabs')
            ),

            'jquery-serialize-object-js' => array(
                'src' => h\assets('js/jquery.serialize-object.min.js'),
            ),

            'lodash-js' => array(
                'src' => h\assets('js/lodash.min.js'),
                'footer' => false
            ),

            'tinyscrollbar-js' => array(
                'src' => h\assets('js/jquery.tinyscrollbar.min.js'),
            ),

            'builder-js' => array(
                'src' => h\assets('js/builder.js'),
            ),

            'gmap-js' => array(
                'src' => h\assets('js/gmap.js') . '?key=' . get_option('qcontact_gmaps_settings_gmap_key', 'AIzaSyB93ROk1PpQBCrOgPV4PKeYp6ECwCk9m_E')
            ),

            'g-captcha' => array(
                'src' => h\assets('js/api.js'),
            ),

            'geolocator' => array(
                'src' => h\assets('js/geolocator.min.js'),
            ),

            'front-js' => array(
                'src' => h\assets('js/custom/front_end.js'),
            ),

            'back-js' => array(
                'src' => h\assets('js/custom/back_end.js'),
            ),

            'general-js' => array(
                'src' => h\assets('js/custom/general.js'),
                'footer' => false
            ),
        ),

        /** Include admin assets . */
        'admin' => array(
            'bootstrap-js', 'tinyscrollbar-js', 'jquery-serialize-object-js', 'lodash-js', 'builder-js', 'back-css', 'gmap-js', 'g-captcha', 'general-js', 'back-js', 'geolocator'
        ),

        /** Include front assets . */
        'front' => array(
         'front-css', 'gmap-js', 'g-captcha', 'front-js',  'lodash-js', 'general-js',
        ),
    ),

    /**
     * Declare media buttons .
     *
     */
    'media' => array(
        'q_function' => array(
            'handler' => function() {
                //
            },
            'view' => function($nonce, $function) {
                return h\view('media', array(

                    'nonce' => $nonce,
                    'function' => $function,
                ));
            }
        )
    ),

    /**
     * Declare all custom post types .
     *
     */
    'post_types' => array(
        'qcontact_inbox' => array(
            'labels' => array(
                'name' => __('QContact', 'q-contact'),
                'singular_name' => __('Inbox', 'q-contact'),
            ),
            'public' => false,
            'menu_icon' => function(){
                return sprintf(
                    '%s/wp-content/plugins/qcontact/assets/images/icon-01.png',
                    get_site_url()
                );
            },
            'show_in_nav_menus' => true,
            'show_ui'      => true,
            'show_in_menu' => true,
            'map_meta_cap' => true,
            'capabilities' => array(
                'create_posts' => false
            ),
            'hierarchical' => true,
            'supports' => ['title', 'editor'],
            'pages' => true,
            'has_archive' => true,
        )
    ),

    /**
     * Declare actions.
     *
     */
    'actions' => array(
        /**
         * That action is used for send email from front end .
         *
         */
        'qcontact_form' => function() {
            try {
                if( isset($_POST['template']) ) {

                    global $wpdb;

                    $table = $wpdb->prefix . 'posts';
                    $template = $wpdb->get_row(
                        sprintf('SELECT * FROM %s WHERE ID = %d', $table, (int) base64_decode($_POST['template']))
                    );

                    if(! $template->ID)
                        throw new ValidationException(
                            __('Invalid template!', 'q-contact')
                        );

                    /**
                     * Check if template require captcha validation .
                     *
                     */
                    if ( (int)$template->post_content_filtered ) {

                        if(! isset($_POST['g-recaptcha-response']))
                            throw new ValidationException(
                                __('Invalid captcha', 'q-contact')
                            );

                        $secret_key = get_option('qcontact_recaptcha_settings_secret_key');

                        if(! $secret_key) {
                            if( is_admin() )
                                throw new ValidationException(
                                    __('Invalid secret key!', 'q-contact')
                                );
                            else
                                throw new ValidationException(
                                    __('Invalid captcha', 'q-contact')
                                );
                        }

                        $recaptcha = new ReCaptcha(
                            get_option('qcontact_recaptcha_settings_secret_key')
                        );

                        $response = $recaptcha->verify(
                            $_POST['g-recaptcha-response']
                        );

                        if(! $response->isSuccess()) {
                            if( is_admin() )
                                throw new ValidationException(
                                    array_pop($response->getErrorCodes())
                                );
                             else
                                throw new ValidationException(
                                    __('Invalid captcha', 'q-contact')
                                );
                        }
                    }

                    /** Clear some of inputs */
                    $to_clear = array('action', 'submit', 'template', 'g-recaptcha-response');
                    foreach ($to_clear as $val)
                        if( isset($_POST[$val]) ) unset($_POST[$val]);

                    if( get_option( 'qcontact_additional_settings_send_to_email', h\settings('qcontact_additional_settings_send_to_email')) ) {
                        if(! $to = get_option('qcontact_settings_mail_to', h\settings('qcontact_settings_mail_to')))
                            $to = get_option('admin_email');

                        if(! $to)
                            throw new ValidationException(
                                __('Invalid email!', 'q-contact')
                            );

                        $subject = get_option('qcontact_settings_mail_subject', h\settings('qcontact_settings_mail_subject'));

                        $message = get_option('qcontact_settings_mail_message', h\settings('qcontact_settings_mail_message'));

                        $content = '';
                        array_walk($_POST, function($v, $k) use(& $content){
                            $content .= sprintf("<strong>%s</strong>: %s \n", ucfirst($k), $v);
                        });

                        $message = str_replace('[message]', $content, $message);

                        wp_mail($to, $subject, $message);
                    }

                    $wpdb->insert($table, array(
                        'post_title' => sprintf("From %s, email %s", $_POST['name'], $_POST['email']),
                        'post_content' => serialize($_POST),
                        'post_author' => get_current_user_id(),
                        'post_status' => 'publish',
                        'comment_status' => 'closed',
                        'ping_status' => 'closed',
                        'post_type' => 'qcontact_inbox',
                    ));

                    $successfullyMessage = get_option('qcontact_messages_successful', h\settings('qcontact_messages_successful'));

                    h\notify(
                        $successfullyMessage,
                        'success'
                    );

                    return wp_redirect(
                        $_SERVER['HTTP_REFERER']
                    );
                }

                $errorMessage = get_option('qcontact_messages_failed', h\settings('qcontact_messages_failed'));

                throw new ValidationException(
                    $errorMessage
                );

            } catch(ValidationException $e) {
                h\notify(
                    $e->getMessage(),
                    'errors'
                );

                return wp_redirect(
                    $_SERVER['HTTP_REFERER']
                );
            }
        },

        /**
         * That action is used for load specific template .
         *
         */
        'load_template' => function() {
            global $wpdb;

            $table = $wpdb->prefix . 'posts';

            $templates = $wpdb->get_results(
                sprintf(
                    'SELECT * FROM %s WHERE post_type = \'%s\' AND post_parent = %s',$table, h\settings('template_type'), $_POST['post_id']
                )
            );

            $original = h\app('template')->get('templates');
            $widgets  = h\app('template')->get('widgets');

            array_walk($templates, function($template) use (& $original){
                if($original[$template->post_name]){
                    $original[$template->post_name]['content'] =  htmlspecialchars_decode(
                        stripslashes($template->post_content)
                    );
                }
            });

            echo h\view('main', array(
                'templates' => $original,
                'widgets' => $widgets,
            ));
        },

        /**
         * That action is used for save specific template .
         *
         */
        'save_template' => function() {
            global $wpdb;

            $table = $wpdb->prefix . 'posts';
            $title = $_POST['title'];
            $post_id = $_POST['post_id'];

            $template = $wpdb->get_row(
                sprintf(
                    'SELECT * FROM %s WHERE post_parent = %s AND post_name=\'%s\'', $table, $post_id, $title
                )
            );

            if($template) {
                $wpdb->update( $table, array(
                    'post_content' => htmlspecialchars($_POST['content']),
                    'post_content_filtered' => (isset($_POST['require_captcha']) && $_POST['require_captcha'] == 1) ? true : false,
                ), array('post_parent' => $_POST['post_id'], 'post_name' => $_POST['title']));
                $stat = 'updated';

                $last_id = $template->ID;
            } else {
                $wpdb->insert( $table, array(
                    'post_content' => htmlspecialchars($_POST['content']),
                    'post_author' => get_current_user_id(),
                    'post_status' => 'inherit',
                    'comment_status' => 'closed',
                    'ping_status' => 'closed',
                    'post_type' => 'qcontact_template',
                    'post_parent' => $_POST['post_id'],
                    'post_content_filtered' => (isset($_POST['require_captcha']) && $_POST['require_captcha'] == 1) ? true : false,
                    'post_title' => ucfirst(str_replace("_", " ", $_POST['title'])),
                    'post_name' => $_POST['title']
                ));

                $stat = 'created';
                $last_id = $wpdb->insert_id;
            }

            echo json_encode(array(
                'id' => $last_id,
                'stat' => $stat
            ));
        }
    ),

    /** Example of shortcode */
    'shortcodes' => array(
        'qcontact' => function($attributes) {
            global $wpdb;

            $table = $wpdb->prefix . 'posts';

            $template = $wpdb->get_row(
                sprintf(
                    'SELECT * FROM %s WHERE ID = %s', $table, $attributes['id']
                )
            );

            $content = htmlspecialchars_decode(
                stripslashes($template->post_content)
            );

            $content = str_replace('</form>', '<input type="hidden" name="template" value="'.base64_encode($template->ID).'"></form>', $content);
            $content = str_replace('data-sitekey=""', 'data-sitekey="'.get_option('qcontact_recaptcha_settings_site_key').'"', $content);

            $message = ((bool) h\notify()->hasErrors()) ?
                h\notify()->errors(true) :
                h\notify()->success(true);

            return $message ? str_replace('<div style="display: none">%message%</div>', $message->render(), $content) : $content;
        }
    ),

    /**
     * Example of shortcodes .
     *
     */
    'filters' => [
        'content_edit_pre' => function($content){
            global $post;

            if($post->post_type == h\settings('inbox_type')){
                $content = '';

                $postContent = unserialize($post->post_content);

                array_walk($postContent, function($v, $k) use(&$content) {
                    if(is_array($v)) $v = implode(', ', $v);

                    $content .= sprintf("<strong>%s</strong>: %s \n", ucfirst($k), $v);
                });

                return $content;
            }

            return $content;
        }
    ]
);