<?php

namespace Builder\Includes;

use Builder\App\Application\Bootstrap\Helpers as h;

class Assets {

    /**
     * @var array
     */
    private $assets;

    public function __construct($assets = array()) {
        $this->setAssets(
            $assets
        );
    }

    /**
     * @param array $assets
     */
    public function setAssets($assets) {
        $this->assets = $assets;
    }

    /**
     * Register all assets to wordPress core API .
     *
     * @return $this
     */
    public function registerAliases() {
        $assets = $this->assets['aliases'];

        array_walk($assets, function($val, $k) {
            if( ! is_array($val) )
                $val = ['src' => $val];

            $params = array_merge($val, [
                'handle' => $k,
            ]);

            $schema    = parse_url($params['src']);
            $filename  = pathinfo($schema['path']);
            $extension = $filename['extension'];


            $function = 'wp_register_script';
            if( $extension == 'css' )
                $function = 'wp_register_style';
            elseif( $extension == 'js' )
                $function = 'wp_register_script';

            $function(
                $params['handle'],
                $params['src'],
                isset($params['dep']) ? $params['dep'] : [],
                isset($params['version']) ? $params['version'] : null,
                $extension == 'css' ? (isset($params['media']) ? $params['media'] : 'all') : (isset($params['footer']) ? $params['footer'] : true)
            );
        });

        return $this;
    }


    /**
     * Return front aliases .
     *
     * @return array
     */
    public function getFrontAliases() {
        return isset($this->assets['front']) ? $this->assets['front'] : [];
    }

    /**
     * Return admin aliases .
     *
     * @return array
     */
    public function getAdminAliases() {
        return isset($this->assets['admin']) ? $this->assets['admin'] : [];
    }


    /**
     * Enqueue aliases .
     *
     * @param array $aliases
     */
    public function enqueue($aliases = array()) {
        array_walk($aliases, function($val, $k) {
            $isGranted = true;

            if( $val instanceof \Closure )
                $isGranted = $val();
            else
                $k = $val;

            if( ! $isGranted )
                return false;

            $asset = isset($this->assets['aliases'][$k]) ? $this->assets['aliases'][$k] : [];
            if(! $asset) return false;

            $file = ! is_array($asset) ? $asset : $asset['src'];

            $file      = pathinfo($file);
            $schema    = parse_url($file['basename']);
            $filename  = pathinfo($schema['path']);
            $extension = $filename['extension'];

            $function = ($extension == 'js') ? 'wp_enqueue_script' : 'wp_enqueue_style';

            $function($k);
        });
    }
}