<?php

namespace Builder\Includes;

use Builder\App\Application\Bootstrap\Helpers as h;

class Fields {

    /**
     * @var array
     */
    protected $data;

    /**
     * @var string
     */
    protected $path = 'elements';

    /**
     * @param array $data
     */
    public function __construct($data = array()) {

        $this->setData($data);
    }


    /**
     * @return array
     */
    public function getData() {
        return $this->data;
    }

    /**
     * @param array $data
     */
    public function setData($data) {
        $this->data = $data;
    }

    /**
     * Register fields and sections to wordPress ..
     *
     * @return $this
     */
    public function register() {
        $sections = $this->data;

        /** Render sections . */
        array_walk($sections, function($value, $key) {
            $label = isset($value['label']) ? $value['label'] : str_replace('-', ' ', $key);
            $section  = $key;

            if(! isset($value['function']))
                $callback = create_function('', 'echo "'.str_replace('"', '\"', $label).'";');
            else
                $callback = $value['function'];

            add_settings_section($section, $label, $callback, $section);

            $fields   = isset($this->data[$section]['fields']) ? $this->data[$section]['fields'] : [];

            /** Render fields . */
            array_walk($fields, function($val, $key) use($section) {

                $label   = isset($val['label']) ? $val['label'] : str_replace('-', ' ', $key);

                $name    = isset($val['name']) ? strtolower($section) . '_' . $val['name'] : strtolower($section) . '_' . $key;

                $value   = ($text = get_option($name)) ? $text : ( ($val['value'] instanceof \Closure) ? $val['value']() : $val['value']);

                $args = [
                    'name' => $name,
                    'section' => $section,
                    'value' => $value,
                    'label' => $label,
                    'options' => isset($val['options']) ? $val['options'] : [],
                    'attr' => isset($val['attr']) ? call_user_func(function() use($val) {
                        $st = '';
                        array_walk($val['attr'], function($v, $k) use(& $st) {
                            $st .= !is_numeric($k) ? $k . '="' . $v . '" ' : $v . ' ';
                        });

                        return $st;
                    }) : '',
                    'size' => isset($val['size']) ? $val['size'] : null,
                    'callback' => isset($val['callback']) ? $val['callback'] : '',
                    'type' => isset($val['type']) ? $val['type'] : 'text',
                    'std' => isset($val['default']) ? $val['default'] : '',
                    'sanitize_callback' => isset($val['sanitize']) ? $val['sanitize'] : '',
                ];

                register_setting($section, $name/*, array($this, 'sanitize_options')*/);
                add_settings_field($name, $label, array($this, 'input'), $section, $section, $args);
            });
        });

        return $this;
    }


    /**
     * Render sections and fields .
     * @param $sections
     */
    public function render($sections = array()) {
        if(! is_array($sections)) $sections = (array)$sections;

        $data_s = array();
        foreach ($sections as $section) {
            if( isset($this->data[$section]) )
                $data_s[$section] = $this->data[$section];
        }

        echo h\view('form', [
            'sections' => $data_s,
        ]);
    }


    /**
     * Render input .
     *
     * @param $args
     * @return mixed
     */
    public function input($args) {
        $value = esc_attr(get_option($args['name'], $args['std']));
        $type  = isset($args['type']) ? $args['type'] : 'text';
        $args  = $args + ['value' => $value];

        if( ! isset($args['type']) && $args['function'] ) {
            $callback = $args['function'];
            $html = $callback($args);
        } else {
            $html = h\view(
                sprintf('%s/%s', $this->path, $type),
                $args
            );
        }

        echo $html;
    }
}