<?php

namespace Builder\Includes;

use Builder\App\Application\Bootstrap\Helpers as h;

class Notify {

    /**
     * Session storage.
     *
     */
    protected $session;

    const FLASH_MESSAGE_PREFIX = 'flash';

    public function __construct() {
        if (! session_id()) session_start();

        if(! isset($_SESSION[self::FLASH_MESSAGE_PREFIX]))
            $_SESSION[self::FLASH_MESSAGE_PREFIX] = array();
    }


    /**
     * Flash a message.
     *
     * @param  string $message
     * @param  string $type
     * @param  array $options
     *
     * @return $this
     */
    public function flash($message, $type = null, array $options = []) {
        $_SESSION[self::FLASH_MESSAGE_PREFIX][$type][] = new Message(
            $message, $type, $options
        );

        return $this;
    }


    /**
     * Get success messages .
     *
     * @param bool $first
     * @return mixed
     */
    public function success($first = true) {
        return $this->get('success', $first);
    }

    /**
     * Get errors messages .
     *
     * @param bool $first
     * @return mixed
     */
    public function errors($first = true) {
        return $this->get('errors', $first);
    }

    /**
     * Check if has errorss
     *
     * @return bool
     */
    public function hasErrors() {
        return $this->has('errors');
    }

    /**
     * Check if has messages .
     *
     * @param $type
     * @return bool
     */
    protected function has($type) {
        return isset($_SESSION[self::FLASH_MESSAGE_PREFIX][$type]);
    }

    /**
     * Return message by type .
     *
     * @param $type
     * @param $first
     * @return mixed
     */
    public function get($type, $first = true) {
        if( $this->has($type) ) {

            $messages = $_SESSION[self::FLASH_MESSAGE_PREFIX][$type];

            $this->clear($type);

            return $first ? array_pop($messages) : $messages;
        }

        return false;
    }

    /**
     * Clear flash types message from session .
     *
     * @param array $types
     * @return $this
     */
    protected function clear($types = array()) {
        if(! is_array($types))
            $types = (array)$types;

        foreach ($types as $type)
            unset($_SESSION[self::FLASH_MESSAGE_PREFIX][$type]);

        return $this;
    }

}