<?php

namespace Builder\Includes;

use Builder\App\Application\Bootstrap\Helpers as h;

class Settings {

    /**
     * The class working array
     * @var array
     */
    protected $settings;

    /**
     * Create the Setting instance
     * @param array $settings
     */
    public function __construct(array $settings = array()) {
        $this->settings = $settings;
    }

    /**
     * Get a value and return it
     * @param string $key String using dot notation
     * @param Mixed $default
     * @return Mixed             The value(s) found
     */
    public function get($key = null, $default = null) {
        if ( empty($key) )
            return $this->settings;

        if( isset($this->settings[$key]) )
            return $this->settings[$key];

        return $default;
    }

    /**
     * Store the passed value into current instance .
     * @param $key
     * @param  mixed $value The value(s) to be stored
     * @return $this
     */
    public function set($key, $value) {
        $this->settings[$key] = $value;

        return $this;
    }

    /**
     * Forget the value(s) currently stored
     * @param  mixed $deleteKey The value(s) to be removed (dot notation)
     * @return $this
     */
    public function forget($deleteKey) {
        if( $this->has($deleteKey) )
            unset($this->settings[$deleteKey]);

        return $this;
    }

    /**
     * Check to see if the value exists
     * @param  string $searchKey The key to search for
     * @return boolean            True: found - False not found
     */
    public function has($searchKey) {
        return isset($this->settings[$searchKey]);
    }
}
