<?php

namespace Builder\Includes;

use Exception;
use Builder\App\Application\Bootstrap\Helpers as h;

class Template {

    /**
     * @var array
     */
    private $paths = array();

    /**
     * @var array
     */
    private $headers = array();

    /**
     * @var array
     */
    public $globals = array();

    /**
     * @var
     */
    protected $ext;

    /**
     * @param $base
     * @param null $globals
     * @param string $ext
     */
    public function __construct($base, $globals = null, $ext = ".tpl.php") {
        $this->paths[] = realpath($base);
        $this->ext = $ext;

        $this->globals = (array)$globals;
    }

    /**
     * Get http header .
     *
     */
    public function httpHeader() {
        $args = func_get_args();
        $this->headers[] = $args;
    }

    /**
     * Apply headers
     *
     */
    public function applyHeaders() {
        foreach ($this->headers as $header) {
            call_user_func_array("header", $header);
        }
    }

    /**
     * @param $name
     * @param $arguments
     * @return string
     * @throws Exception
     */
    public function __call($name, $arguments) {
        $paths = $this->paths;

        foreach ($paths as $path) {
            $file = $path . DIRECTORY_SEPARATOR . ($name) . $this->ext;

            if ( file_exists($file) ) {

                if (count($arguments) === 0)
                    $arguments[0] = array();

                $args = array_merge((array)$this->globals, $arguments[0]);

                ob_start("mb_output_handler");

                extract($args);

                include $file;

                $content = ob_get_contents();
                ob_end_clean();

                return $content;
            }
        }

        throw new Exception("File not found ($name)");
    }
}